# Generated by roxygen2: do not edit by hand

S3method(as_ggwebgl_spec,default)
S3method(as_ggwebgl_spec,ggwebgl_spec)
S3method(as_ggwebgl_spec,xgeo_state)
S3method(as_mesh_webgl,data.frame)
S3method(as_mesh_webgl,default)
S3method(as_mesh_webgl,ggwebgl_mesh)
S3method(ggplot2::ggplot_add,ggwebgl_coord_3d)
S3method(ggplot2::ggplot_add,ggwebgl_theme)
S3method(ggplot2::ggplot_add,ggwebgl_time_scale)
export(animation_spec)
export(as_ggwebgl_spec)
export(as_mesh_webgl)
export(compose_ggwebgl_figure)
export(coord_webgl_3d)
export(geom_area_webgl)
export(geom_bar_webgl)
export(geom_bin2d_webgl)
export(geom_boxplot_webgl)
export(geom_contour_webgl)
export(geom_crossbar_webgl)
export(geom_density2d_webgl)
export(geom_density_webgl)
export(geom_errorbar_webgl)
export(geom_freqpoly_webgl)
export(geom_histogram_webgl)
export(geom_label_webgl)
export(geom_line_webgl)
export(geom_linerange_webgl)
export(geom_mesh_webgl)
export(geom_path3d_webgl)
export(geom_path_webgl)
export(geom_point_webgl)
export(geom_pointrange_webgl)
export(geom_polygon_webgl)
export(geom_raster_webgl)
export(geom_rect_webgl)
export(geom_ribbon_webgl)
export(geom_rug_webgl)
export(geom_segment_webgl)
export(geom_surface_webgl)
export(geom_text_webgl)
export(geom_tile_webgl)
export(geom_vector_webgl)
export(geom_violin_webgl)
export(ggWebGL)
export(ggWebGLOutput)
export(ggplot_webgl)
export(ggwebgl_example_data)
export(ggwebgl_interactions)
export(ggwebgl_layer_lines)
export(ggwebgl_layer_mesh)
export(ggwebgl_layer_points)
export(ggwebgl_layer_raster)
export(ggwebgl_layer_surface)
export(ggwebgl_layer_vectors)
export(ggwebgl_magnify_region)
export(ggwebgl_material)
export(ggwebgl_mesh)
export(ggwebgl_publication_figure)
export(ggwebgl_selection)
export(ggwebgl_spec)
export(ggwebgl_timeline)
export(ggwebgl_transport)
export(ggwebgl_view)
export(renderGgWebGL)
export(scale_time_webgl)
export(snapshot_ggwebgl)
export(stat_surface_webgl)
export(surface_matrix)
export(theme_webgl)
export(updateGgWebGLTimeline)
export(webgl_spec)
importFrom(utils,getS3method)
